
from pyee.base import EventEmitter

from pyfabric.Networking.Client import Client
from pyfabric.World.World import ClientWorld
from pyfabric.Events.EventType import EventType
from pyfabric.Entity.Entity import Player, Entity


class Fabric(EventEmitter):
    def __init__(self):
        super().__init__()
        self.client = Client(self.__handle_event__, 1337)
        self.cmd = EventEmitter()
        self.__client_player__ = None
        self.__ready__ = False
        self.queue_event = []

    def __handle_event__(self, event_type: str, packet: dict):
        if not self.__ready__:
            self.queue_event.append((event_type, packet))
            return

        val = EventType(event_type)
        if val == EventType.ON_COMMAND:
            data = packet['data']
            if len(data) == 1:
                self.cmd.emit("/" + data[0][1:], self.get_client_player())
            else:
                self.cmd.emit("/" + data[0][1:], self.get_client_player(), data[1:])
        self.emit(val)

    def ready(self):
        self.__ready__ = True
        for event_type, packet in self.queue_event:
            self.__handle_event__(event_type, packet)

        self.queue_event.clear()

    def get_client_player(self):
        if not self.client.connected:
            return None

        if self.__client_player__ is None:
            uuid = self.client.request_string("get_client_player_uuid", 0)
            self.__client_player__ = Player(self.client, uuid)

        return self.__client_player__

    def get_player(self, uuid: str):
        if not self.client.connected:
            return None
        return Player(self.client, uuid)

    def get_entity(self, uuid):
        if not self.client.connected:
            return None
        return Entity(self.client, uuid)

    def get_world(self):
        if not self.client.connected:
            return None
        return ClientWorld(self.client)

    def register_command(self, cmd: str):
        self.client.notify_server("register_command", cmd)


def create():
    return Fabric()




from src.pyfabric.Command import CommandBuilder, ArgumentType
import src.pyfabric.Fabric as Fabric
from src.pyfabric.EventType import EventType

Minecraft = Fabric.create()

# Register commands
Minecraft.register_command(CommandBuilder()
                           .set_name("python")
                           .add_sub_command(CommandBuilder()
                                            .set_name("armor")
                                            .build())
                           .add_sub_command(CommandBuilder()
                                            .set_name("hand")
                                            .add_sub_command(CommandBuilder()
                                                             .set_name("main")
                                                             .build())
                                            .add_sub_command(CommandBuilder()
                                                             .set_name("off")
                                                             .build())
                                            .build())
                           .add_sub_command(CommandBuilder()
                                            .set_name("slot")
                                            .add_sub_command(CommandBuilder()
                                                             .set_name("item_slot")
                                                             .set_arg(ArgumentType.INTEGER)
                                                             .build())
                                            .build())
                           .build())


@Minecraft.on(EventType.JOIN)
def join_world():
    print("Loaded world")

    player = Minecraft.get_client_player()

    player.send_chat("§ePython§7 connect with §6your§7 client")  # Limited to max chat limit

    # player.send_actionbar(f"§aYour world: {player.get_dimension()}")
    # print(player.get_inventory().get_mainhand())
    # print(Minecraft.get_world().get_block_at(0, 70, 0))


# Handle commands
@Minecraft.cmd.on("/python hand main")
def tell_main_item(player):
    main_hand = player.get_inventory().get_mainhand()
    player.send_chat(main_hand.registry)


@Minecraft.cmd.on("/python hand off")
def tell_off_item(player, args):
    main_hand = player.get_inventory().get_offhand()
    player.send_chat(main_hand.registry)


@Minecraft.cmd.on("/python armor")
def tell_armor(player, args):
    armor_generator = player.get_inventory().get_armor()
    for armor in list(armor_generator):
        player.send_chat(armor.registry)


@Minecraft.cmd.on("/python slot")
def tell_slot(player, args):
    player.send_chat(args[0])


Minecraft.ready()
